@@ -2,6 +2,7 @@ import unittest
 from typing import Any, List
 
 import pymich.instr_types as t
+import pymich.exceptions as E
 from pymich.vm_types import Contract, Instr, Or, Pair
 
 
@@ -22,6 +23,10 @@ class CompilerBackend:
             micheline = {
                 "prim": "int",
             }
+        elif type(parameter) == t.Nat:
+            micheline = {
+                "prim": "nat",
+            }
         elif type(parameter) == t.Mutez:
             micheline = {
                 "prim": "mutez",
@@ -50,6 +55,14 @@ class CompilerBackend:
                     self.compile_type(parameter.cdr),
                 ],
             }
+        elif type(parameter) == t.Pair:
+            micheline = {
+                "prim": "pair",
+                "args": [
+                    self.compile_type(parameter.car),
+                    self.compile_type(parameter.cdr),
+                ],
+            }
         elif type(parameter) == t.Dict:
             micheline = {
                 "prim": "map",
@@ -73,9 +86,26 @@ class CompilerBackend:
                     self.compile_type(parameter.param_type),
                 ]
             }
+        elif type(parameter) == t.List:
+            micheline = {
+                "prim": "list",
+                "args": [
+                    self.compile_type(parameter.element_type),
+                ]
+            }
+        elif type(parameter) == t.Unit:
+            micheline = {
+                "prim": "unit",
+            }
+        elif type(parameter) == t.Operation:
+            micheline = {
+                "prim": "operation",
+            }
+        elif type(parameter) == t.Record:
+            micheline = self.compile_type(parameter.get_type())
         else:
             breakpoint()
-            return NotImplementedError
+            raise E.CompilerException(f"{parameter} is missing in the backend compiler", 0)
 
         try:
             if parameter.annotation:
@@ -115,7 +145,7 @@ class CompilerBackend:
         elif instruction.name == "DUP":
             return {"prim": "DUP"}
         elif instruction.name == "NIL":
-            return {"prim": "NIL", "args": [{"prim": str(instruction.args[0])}]}
+            return {"prim": "NIL", "args": [self.compile_type(instruction.args[0])]}
         elif instruction.name == "LIST":
             return NotImplementedError
         elif instruction.name == "PAIR":
@@ -216,6 +246,13 @@ class CompilerBackend:
                     self.compile_instructions(instruction.args[1]),
                 ],
             }
+        elif instruction.name == "ITER":
+            return {
+                "prim": "ITER",
+                "args": [
+                    self.compile_instructions(instruction.args[0]),
+                ],
+            }
         else:
             breakpoint()
             raise NotImplementedError
@@ -251,28 +288,3 @@ class CompilerBackend:
             },
         ]
 
-
-class TestCompilerBackend(unittest.TestCase):
-    def test_compile_param(self):
-        b = CompilerBackend()
-        compiled_parameter = b.compile_type(Or(t.Int(), t.Int()))
-        expected_parameter = {
-            "prim": "or",
-            "args": [
-                {
-                    "prim": "int",
-                },
-                {
-                    "prim": "int",
-                },
-            ],
-        }
-        self.assertEqual(compiled_parameter, expected_parameter)
-
-
-for TestSuite in [
-    TestCompilerBackend,
-]:
-    suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestSuite)
-    unittest.TextTestRunner().run(suite)
-
