@@ -4,6 +4,8 @@ from dataclasses import dataclass
 
 import unittest
 
+import pymich.exceptions as E
+
 
 
 def _is_view(node) -> bool:
@@ -25,9 +27,10 @@ def make_dataclass(name, arguments_spec):
         body=[
             ast.AnnAssign(
                 target=ast.Name(id=argument_name, ctx=ast.Store()),
-                annotation=argument_type,
+                annotation=argument_type[0],
                 value=None,
                 simple=1,
+                lineno=argument_type[1],
             )
             for argument_name, argument_type in arguments_spec.items()
         ],
@@ -70,10 +73,6 @@ class RewriteViews(ast.NodeTransformer):
         for node in nodes:
             if type(node) == ast.Return:
                 new_nodes.append(self._transform_return(node))
-            #elif type(node) == ast.If:
-            #    raise NotImplementedError
-            #elif type(node) == ast.For:
-            #    raise NotImplementedError
             else:
                 new_nodes.append(node)
 
@@ -155,7 +154,7 @@ class TuplifyFunctionArguments(ast.NodeTransformer):
         if len(arguments) > 1:
             ### generate argument dataclass
             arguments_spec = {
-                argument_node.arg: argument_node.annotation
+                argument_node.arg: (argument_node.annotation, node.lineno)
                 for argument_node in arguments
             }
             param_dataclass_name = node.name + "Param"
@@ -191,8 +190,12 @@ class TuplifyFunctionArguments(ast.NodeTransformer):
         if type(node.func) == ast.Attribute and node.func.attr == "get":
             return node
 
+        if type(node.func) == ast.Attribute:
+            raise E.CompilerException("Functions as attributes are not yet supported", node.func.lineno)
         fun_name = node.func.id
         if len(node.args) > 1 and fun_name not in self.defined_class_names and fun_name != "transaction":
+            if fun_name not in self.env:
+                raise E.FunctionNameException(fun_name, node.lineno)
             node.args = [
                 ast.Call(
                     func=ast.Name(id=self.env[fun_name], ctx=ast.Load()),
@@ -310,7 +313,7 @@ class FactorOutStorage(ast.NodeTransformer):
         if node.name != "Contract":
             return node
 
-        # if there is a `deploy` method, do nothing
+        # TODO remove: if there is a `deploy` method, do nothing
         for body_node in node.body:
             if type(body_node) == ast.FunctionDef:
                 if body_node.name == "deploy":
@@ -321,7 +324,8 @@ class FactorOutStorage(ast.NodeTransformer):
         new_node_body = []
         for i, body_node in enumerate(node.body):
             if type(body_node) == ast.AnnAssign:
-                storage_keys_spec[body_node.target.id] = body_node.annotation
+                lineno = body_node.lineno
+                storage_keys_spec[body_node.target.id] = (body_node.annotation, lineno)
             else:
                 new_node_body.append(body_node)
         node.body = new_node_body
@@ -391,127 +395,3 @@ def macro_expander(source_ast):
         new_ast = PlaceBackStorageDataclass(pass2.storage_dataclass).visit(new_ast)
     return ast.fix_missing_locations(new_ast)
 
-
-class TestRewriteViews(unittest.TestCase):
-    def test_function_call_in_block(self):
-        source = """
-class Contract:
-    def myView(self, arg1: type1, arg2: type2) -> return_type:
-        return f(arg1, arg2)
-        """
-        source_ast = ast.parse(source)
-        new_ast = RewriteViews().visit(source_ast)
-        new_ast = ast.fix_missing_locations(new_ast)
-        view_ast = new_ast.body[0].body[0]
-
-        # test function prototype
-        self.assertEqual(view_ast.returns, None)
-        last_argument = view_ast.args.args[-1]
-        self.assertEqual(last_argument.arg, '__callback__')
-        self.assertEqual(type(last_argument.annotation), ast.Subscript)
-        self.assertEqual(type(last_argument.annotation.value), ast.Name)
-        self.assertEqual(last_argument.annotation.value.id, 'Contract')
-        self.assertEqual(type(last_argument.annotation.slice), ast.Name)
-        self.assertEqual(last_argument.annotation.slice.id, 'return_type')
-
-        # test transaction call
-        transaction = view_ast.body[-1]
-        self.assertEqual(transaction.func.id, "transaction")
-        self.assertEqual(type(transaction.args[0]), ast.Name)
-        self.assertEqual(transaction.args[0].id, "__callback__")
-        self.assertEqual(type(transaction.args[1]), ast.Call)
-        self.assertEqual(transaction.args[1].func.id, "mutez")
-        self.assertEqual(type(transaction.args[1].args[0]), ast.Constant)
-        self.assertEqual(transaction.args[1].args[0].value, 0)
-        return_value = ast.parse(source).body[0].body[0].body[0].value
-        self.assertEqual(ast.dump(transaction.args[2]), ast.dump(return_value))
-
-
-class TestFactorOutStorage(unittest.TestCase):
-    def test_new_function_evaluates(self):
-        source = """
-class Contract:
-    counter: int
-    admin: str
-
-    def update_counter(self, new_counter: int) -> int:
-        self.counter = 1
-
-    def update_admin(self, new_admin: str) -> int:
-        self.admin = new_admin
-        """
-        source_ast = ast.parse(source)
-        expander = FactorOutStorage()
-        new_ast = expander.visit(source_ast)
-        new_ast.body = [expander.storage_dataclass] + new_ast.body
-        new_ast = ast.fix_missing_locations(new_ast)
-
-
-class TestRemoveSelfArgFromMethods(unittest.TestCase):
-    def test_new_function_evaluates(self):
-        source = """
-class C:
-    def f(self, x: int, y: str, z: int): return 1
-        """
-        source_ast = ast.parse(source)
-        new_ast = RemoveSelfArgFromMethods().visit(source_ast)
-        new_method_ast = source_ast.body[0].body[0]
-        self.assertEqual(len(new_method_ast.args.args), 3)
-
-        for arg_node, arg_name, arg_type in zip(new_method_ast.args.args, ['x', 'y', 'z'], ['int', 'str', 'int']):
-            self.assertEqual(arg_node.arg, arg_name)
-            self.assertEqual(arg_node.annotation.id, arg_type)
-
-
-class TestTuplifyFunctionArguments(unittest.TestCase):
-    def test_new_function_evaluates(self):
-        source = """
-from dataclasses import dataclass
-
-def add(x: int, y: int, z: int) -> int:
-    return x + y + z
-
-def increment(x: int) -> int:
-    return add(x, 1, 0)
-
-assert add(1, 2, add(3, 4, 5)) == 15
-assert increment(10) == 11
-"""
-        f_ast = ast.parse(source)
-        pass1 = TuplifyFunctionArguments()
-        new_f_ast = pass1.visit(f_ast)
-        new_f_ast.body = new_f_ast.body[:1] + pass1.dataclasses + new_f_ast.body[1:]
-        new_f_ast = ast.fix_missing_locations(new_f_ast)
-        local_vars = {}
-        eval(compile(new_f_ast, '', mode='exec'), local_vars)
-        add = local_vars["add"]
-        addParam = local_vars["addParam"]
-        self.assertEqual(add(addParam(1, 2, add(addParam(3, 4, 5)))), 15)
-
-
-class TestAssignAllFunctionCallsTests(unittest.TestCase):
-    def test_function_call_in_block(self):
-        source = """
-f = lambda x: x
-if True:
-    y = f(1)
-    f(2)
-
-assert __placeholder__ == 2
-assert y == 1
-        """
-        source_ast = ast.parse(source)
-        new_ast = AssignAllFunctionCalls().visit(source_ast)
-        new_ast = ast.fix_missing_locations(new_ast)
-        eval(compile(new_ast, '', mode='exec'))
-
-
-for TestSuite in [
-        TestRewriteViews,
-        TestTuplifyFunctionArguments,
-        TestAssignAllFunctionCallsTests,
-        TestRemoveSelfArgFromMethods,
-        TestFactorOutStorage,
-]:
-    suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestSuite)
-    unittest.TextTestRunner().run(suite)
